local addonName, addon = ...

local plugin = addon:NewModule("Coordinates", "AceHook-3.0")
plugin.Icon = [[Interface\Icons\Achievement_BG_3flagcap_nodeaths]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")
local LSM 		= LibStub("LibSharedMedia-3.0")

local db

local SettingPoistion = false
local ChangingShow = false
local MouseIsOver = false
local ConfigMode = false
local CoordFormat

local Frame = CreateFrame("Frame", "mMinimapCoordinatesFrame", Minimap)
Frame.Text = Frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
Frame.Text:SetAllPoints()
Frame:SetFrameLevel(Minimap:GetFrameLevel() + 5)

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "BlizzMinimap",
			Point = "TOP",
			RelPoint = "TOP",
			x = 0,
			y = -2,
			
			Scale = 1,
			
			Hide = false,
			ShowOnMouseOver = true,
			
			FontSettings = {
				Font = "Default",
				Size = 12,
				Flags = "OUTLINE",
				Color = {1, 1, 1, 1},
			},
			
			Precision = 1,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "Coordinates", "mMinimapCoordinates", Frame)
end

function plugin:OnEnable()		
	addon.RegisterCallback(self, "MinimapEnter")
	addon.RegisterCallback(self, "MinimapLeave")
	addon.RegisterCallback(self, "ConfigMode")
	
	self:ApplySettings()
	
	self:SecureHook(Frame.Text, "SetFormattedText", "SetText")
	self:SecureHook(Frame.Text, "SetFont", "SetText")
	
	Frame:SetScript("OnUpdate", function(self, elapsed)
		self.elapsed = (self.elapsed or 0) + elapsed
		if self.elapsed > 0.2 then
			local x, y = GetPlayerMapPosition("player")
			if x and y then
				Frame.Text:SetFormattedText(CoordFormat, x * 100, y * 100)
			end
			self.elapsed = 0
		end	
	end)
end

function plugin:OnDisable()
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:MinimapEnter()
	MouseIsOver = true
	if not Frame:IsShown() and db.ShowOnMouseOver then
		Frame:Show()
	end
end

function plugin:MinimapLeave()
	MouseIsOver = false
	if Frame:IsShown() and db.Hide and not ConfigMode then
		Frame:Hide()
	end
end

function plugin:SetText()
	Frame:SetWidth(Frame.Text:GetStringWidth())
	Frame:SetHeight(Frame.Text:GetStringHeight())
end

function plugin:ApplySettings()	
	CoordFormat = ("%%.%df, %%.%df"):format(db.Precision, db.Precision)

	Frame.Text:SetFont(LSM:Fetch("font", db.FontSettings.Font), db.FontSettings.Size, db.FontSettings.Flags)
	Frame.Text:SetTextColor(unpack(db.FontSettings.Color))

	Frame:SetScale(db.Scale)
	Frame:ClearAllPoints()
	local anchor = FA:GetAnchor(db.Anchor)
	local s = Frame:GetEffectiveScale()
	Frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	if db.Hide and not ConfigMode then
		Frame:Hide()
	else
		Frame:Show()
	end
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		Frame:Show()
	elseif db.Hide then
		Frame:Hide()
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Hide", db, "Hide", Callback, nil, 0.33))
		g:AddChild(UIF:CheckBox("Show on mouse over", db, "ShowOnMouseOver", Callback, nil, 0.33))
		
		g:AddChild(UIF:NewLine())
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(Frame))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		
		g:AddChild(UIF:NewLine(15))
		
		g:AddChild(UIF:Slider("Coordinate Precision", db, "Precision", 0, 3, 1, Callback, 1))
		
		g:AddChild(UIF:FontSettings("Font settings", db.FontSettings, true, Callback, 1))
		return g
	end
end