local addonName, addon = ...

local plugin = addon:NewModule("Quest Tracker", "AceHook-3.0", "AceEvent-3.0")
plugin.Icon = [[Interface\Calendar\EventNotification]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

local SettingPoistion = false
local ConfigMode = false
local ConfigFrame
local SetPointOutOfCombat = false
local ApplySettingsOutOfCombat = false

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "BlizzMinimapCluster",
			Point = "TOPRIGHT",
			RelPoint = "BOTTOMRIGHT",
			x = 0,
			y = -10,
			
			Scale = 1,
			
			Height = 400,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "Quest Tracker", "mMinimapQuestTracker", WatchFrame)
	
	ConfigFrame = WatchFrame:CreateTexture()
	ConfigFrame:SetTexture(0.5, 0, 0, 0.5)
	ConfigFrame:SetAllPoints()
	ConfigFrame:Hide()
	
	self:SetEnabledState(db.Enabled)
end

function plugin:OnEnable()

	self:SecureHook(WatchFrame, "SetPoint", "SetPosition")
	
	addon.RegisterCallback(self, "ConfigMode")
	FA.RegisterCallback(self, "FrameRegistered", "ApplySettings")
	
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	self:ApplySettings()
end

function plugin:OnDisable()
	
	addon.UnregisterAllCallbacks(self)
	FA.UnregisterAllCallbacks(self)
end

function plugin:PLAYER_REGEN_ENABLED()
	if SetPointOutOfCombat then
		WatchFrame:SetPoint("CENTER")
	end
	if ApplySettingsOutOfCombat then
		self:ApplySettings()
	end
	
	SetPointOutOfCombat = false
	ApplySettingsOutOfCombat = false
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:SetPosition(frame)
	if InCombatLockdown() then
		SetPointOutOfCombat = true
		return
	end
	if SettingPoistion then return end
	SettingPoistion = true
	
	frame:SetScale(db.Scale)	
	local anchor = FA:GetAnchor(db.Anchor)
	local s = frame:GetEffectiveScale()
	frame:ClearAllPoints()
	frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	
	SettingPoistion = false
end

function plugin:ApplySettings()	
	if InCombatLockdown() then
		ApplySettingsOutOfCombat = true
		return
	end
	WatchFrame:SetPoint("CENTER")
	WatchFrame:SetHeight(db.Height)
	
	if self:IsEnabled() and not db.Enabled then self:Disable() end
	if not self:IsEnabled() and db.Enabled then self:Enable() end
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		ConfigFrame:Show()
	else
		ConfigFrame:Hide()
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Enabled", db, "Enabled", Callback, "Disable this module if you have a different addon that controls the quest wather to prevent conflicts."))
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(WatchFrame))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		g:AddChild(UIF:Slider("Height", db, "Height", 0, 800, 1, Callback, 1))
		
		return g
	end
end