local addonName, addon = ...

local plugin = addon:NewModule("Tracking", "AceHook-3.0")
plugin.Icon = [[Interface\Minimap\TRACKING\None]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

local SettingPoistion = false
local ConfigMode = false

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "BlizzMinimap",
			Point = "TOPLEFT",
			RelPoint = "TOPLEFT",
			x = 0,
			y = 0,
			
			Scale = 1,
			
			Hide = true,
			ShowOnMouseOver = true,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "Tracking Button", "mMinimapTrackingButton", MiniMapTracking)
end

function plugin:OnEnable()	
	self:SecureHook(MiniMapTracking, "SetPoint", "SetPosition")
	self:HookScript(MiniMapTrackingButton, "OnEnter", function()
		Minimap:GetScript("OnEnter")(Minimap)
	end)
	self:HookScript(MiniMapTrackingButton, "OnLeave", function()
		Minimap:GetScript("OnLeave")(Minimap)
	end)
	
	addon.RegisterCallback(self, "MinimapEnter")
	addon.RegisterCallback(self, "MinimapLeave")
	addon.RegisterCallback(self, "ConfigMode")
	
	self:ApplySettings()
end

function plugin:OnDisable()
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:MinimapEnter()
	if not MiniMapTracking:IsShown() and db.ShowOnMouseOver then
		MiniMapTracking:Show()
	end
end

function plugin:MinimapLeave()
	if MiniMapTracking:IsShown() and db.Hide and not ConfigMode then
		MiniMapTracking:Hide()
	end
end

function plugin:SetPosition()
	if SettingPoistion then return end
	SettingPoistion = true
	MiniMapTracking:SetScale(db.Scale)
	MiniMapTracking:ClearAllPoints()
	local anchor = FA:GetAnchor(db.Anchor)
	local s = MiniMapTracking:GetEffectiveScale()
	MiniMapTracking:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	SettingPoistion = false
end

function plugin:ApplySettings()	
	MiniMapTrackingButtonBorder:Hide()	
	MiniMapTrackingIconOverlay:SetTexture(nil)
	MiniMapTrackingButton:SetHighlightTexture(nil)
	MiniMapTrackingIcon:SetTexCoord(0.065, 0.935, 0.065, 0.935)

	MiniMapTracking:SetPoint(db.Point)
	
	if db.Hide and not ConfigMode then
		MiniMapTracking:Hide()
	else
		MiniMapTracking:Show()
	end
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		MiniMapTracking:Show()
	elseif db.Hide then
		MiniMapTracking:Hide()
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Hide", db, "Hide", Callback, nil, 0.33))
		g:AddChild(UIF:CheckBox("Show on mouse over", db, "ShowOnMouseOver", Callback, nil, 0.33))
		
		g:AddChild(UIF:NewLine())
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(MiniMapTracking))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		return g
	end
end