local addonName, addon = ...

local plugin = addon:NewModule("World Information", "AceHook-3.0")
plugin.Icon = [[Interface\Icons\Achievement_WorldEvent_ChildrensWeek]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

local SettingPoistion = false
local ConfigMode = false
local ConfigFrame

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "",
			Point = "TOP",
			RelPoint = "TOP",
			x = -5,
			y = -15,
			
			Scale = 1,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
		
	ConfigFrame = CreateFrame("Frame", "mMinimapAlwaysUpFrame", WorldStateAlwaysUpFrame, "WorldStateAlwaysUpTemplate")
	mMinimapAlwaysUpFrameText:SetText("Controlled: 2 ")
	mMinimapAlwaysUpFrameIcon:SetTexture([[Interface\WorldStateFrame\HordeTower]])
	mMinimapAlwaysUpFrame:SetPoint("TOP", WorldStateAlwaysUpFrame, -23 , -20)
	
	FA:RegisterAnchor("mMinimap", "World Information", "mMinimapWorldInformation", WorldStateAlwaysUpFrame)
end

function plugin:OnEnable()		
	
	self:SecureHook(WorldStateAlwaysUpFrame, "SetPoint", "SetPosition")
	
	addon.RegisterCallback(self, "ConfigMode")
	FA.RegisterCallback(self, "FrameRegistered", "ApplySettings")
	
	self:ApplySettings()
end

function plugin:OnDisable()
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:SetPosition(frame)
	if SettingPoistion then return end
	SettingPoistion = true
	
	frame:SetScale(db.Scale)	
	local anchor = FA:GetAnchor(db.Anchor)
	local s = frame:GetEffectiveScale()
	frame:ClearAllPoints()
	frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	
	SettingPoistion = false
end

function plugin:ApplySettings()	
	
	WorldStateAlwaysUpFrame:SetPoint("CENTER")
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		ConfigFrame:Show()
	else
		ConfigFrame:Hide()
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(MinimapZoneTextButton))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		
		return g
	end
end