local addonName, addon = ...

local plugin = addon:NewModule("Zone Name", "AceHook-3.0", "AceEvent-3.0")
plugin.Icon = [[Interface\Icons\Achievement_Dungeon_Outland_Dungeon_Hero]]

local AceGUI	= LibStub("AceGUI-3.0")
local LSM		= LibStub("LibSharedMedia-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

local SettingPoistion = false
local ConfigMode = false
local SetPointOutOfCombat = false

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "BlizzMinimap",
			Point = "BOTTOM",
			RelPoint = "TOP",
			x = 0,
			y = 5,
			
			Scale = 1,
			
			Hide = false,
			ShowOnMouseOver = true,
			
			FontSettings = {
				Font = "Default",
				Size = select(2, MinimapZoneText:GetFont()),
				Flags = "OUTLINE",
			}
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "Zone Name", "mMinimapZoneName", MinimapZoneTextButton)
	local t =MinimapZoneTextButton:CreateTexture()
	t:SetTexture(0.5, 0, 0, 0.5)
	--t:SetAllPoints()
	--t:Show()
end

function plugin:OnEnable()	
	MinimapZoneTextButton:SetFrameLevel(Minimap:GetFrameLevel() + 1)
	self:SecureHook(MinimapZoneTextButton, "SetPoint", "SetPosition")
	self:SecureHook(MinimapZoneText, "SetText", "UpdateText")
	
	addon.RegisterCallback(self, "MinimapEnter")
	addon.RegisterCallback(self, "MinimapLeave")
	addon.RegisterCallback(self, "ConfigMode")
	FA.RegisterCallback(self, "FrameRegistered", "ApplySettings")
	
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	self:ApplySettings()
end

function plugin:OnDisable()
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:PLAYER_REGEN_ENABLED()
	if ApplySettingsOutOfCombat then
		self:ApplySettings()
	end
	
	SetPointOutOfCombat = false
end

function plugin:MinimapEnter()
	if db.ShowOnMouseOver and not MinimapZoneTextButton:IsShown() then
		MinimapZoneTextButton:Show()
	end
end

function plugin:MinimapLeave()
	if db.Hide and MinimapZoneTextButton:IsShown() and not ConfigMode then
		MinimapZoneTextButton:Hide()
	end
end

function plugin:UpdateText(text, ...)
	if InCombatLockdown() then
		SetPointOutOfCombat = true
		return
	end
	MinimapZoneTextButton:SetWidth(MinimapZoneText:GetStringWidth())
end

function plugin:SetPosition()
	if InCombatLockdown() then
		SetPointOutOfCombat = true
		return
	end
	if SettingPoistion then return end
	SettingPoistion = true
	MinimapZoneTextButton:SetScale(db.Scale)
	MinimapZoneTextButton:ClearAllPoints()
	local anchor = FA:GetAnchor(db.Anchor)
	local s = MinimapZoneTextButton:GetEffectiveScale()
	MinimapZoneTextButton:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	SettingPoistion = false
end

function plugin:ApplySettings()	
	MinimapZoneText:SetFont(LSM:Fetch("font", db.FontSettings.Font), db.FontSettings.Size, db.FontSettings.Flags)
	MinimapZoneText:SetJustifyH("CENTER")
	MinimapZoneText:SetAllPoints()
	
	MinimapZoneTextButton:SetPoint(db.Point)
	
	if db.Hide and not ConfigMode then
		MinimapZoneTextButton:Hide()
	else
		MinimapZoneTextButton:Show()
	end
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		MinimapZoneTextButton:Show()
	elseif db.Hide then
		MinimapZoneTextButton:Hide()
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	FontFlagList = {
		["NONE"] = "None",
		["OUTLINE"] = "Outline",
		["THICKOUTLINE"] =  "Thick Outline",
		["MONOCHROME"] = "Monochrome",
		["OUTLINEMONOCHROME"] = "Outline Monochrome",
	}
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Hide", db, "Hide", Callback, nil, 0.33))
		g:AddChild(UIF:CheckBox("Show on mouse over", db, "ShowOnMouseOver", Callback, nil, 0.33))
		
		g:AddChild(UIF:NewLine())
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(MinimapZoneTextButton))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		
		g:AddChild(UIF:FontSettings("Font settings", db.FontSettings, false, Callback, 1))
		return g
	end
end