local mMinimap = mMinimap

local db
local FrameAnchors = {}
local MultiFrames = {
	DurabilityFrame = DurabilityFrame,
	VehicleSeatIndicator = VehicleSeatIndicator,
	WatchFrame = WatchFrame,
	TicketStatusFrame = TicketStatusFrame,
	Capture = "Capture",
	WorldStateAlwaysUpFrame = WorldStateAlwaysUpFrame,
}
local FrameRealNames = {
	DurabilityFrame = "Durability Frame",
	VehicleSeatIndicator = "Vehicle Seats",
	WatchFrame = "Quest Tracker",
	TicketStatusFrame = "Ticket Status",
	Capture = "Capture bar",
	WorldStateAlwaysUpFrame = "World Information",
}
local AnchorFrameColor = {0.5, 0.1, 0, 0.5}
local SettingPositions
local NumHookedCaptureBars = 0

mMinimap.FrameRealNames = FrameRealNames
mMinimap.MultiFrames = MultiFrames

function mMinimap:ShowFrameAnchors()
	self = mMinimap
	for frameName, frame in pairs(MultiFrames) do
		if not FrameAnchors[frameName] then
			self:CreateFrameAnchor(frameName)
		end
		FrameAnchors[frameName]:Show()
	end
end

function mMinimap:HideFrameAnchors()
	self = mMinimap
	for frameName, frame in pairs(MultiFrames) do
		if FrameAnchors[frameName] then
			FrameAnchors[frameName]:Hide()
		end		
	end
end

local function FrameAnchor_OnDragStart(self)
	self:StartMoving()
end

local function FrameAnchor_OnDragStop(self)
	self:StopMovingOrSizing()
	
	local x, y = self:GetCenter()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth() / 2
	y = y - GetScreenHeight() / 2
	
	x = x / scale
	y = y / scale
	
	db.FramePositions[self.name] = db.FramePositions[self.name] or {}
	db.FramePositions[self.name][1] = x
	db.FramePositions[self.name][2] = y
	
	mMinimap:SetFramePosition(self.name)
	mMinimap:ToggleConfigIfVisible()
end

function mMinimap:CreateFrameAnchor(frameName)
	local frame = CreateFrame("Frame", "mMinimapFrameAnchor_"..frameName, UIParent)
	if MultiFrames[frameName].SetPoint then
		frame:SetAllPoints(MultiFrames[frameName])
	else
		frame:SetPoint("TOP", Minimap, "BOTTOM", 0, -4)
		frame:SetWidth(Minimap:GetWidth() * Minimap:GetEffectiveScale() / UIParent:GetEffectiveScale())
		frame:SetHeight(35)
	end
	
	if frameName == "WorldStateAlwaysUpFrame" then
		WorldStateAlwaysUpFrame:SetWidth(300)
		WorldStateAlwaysUpFrame:SetHeight(70)
		frame:SetFrameLevel(10)
	end
	
	frame:SetScript("OnDragStart", FrameAnchor_OnDragStart)
	frame:SetScript("OnDragSTop", FrameAnchor_OnDragStop)
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame.name = frameName
	
	frame.texture = frame:CreateTexture()
	frame.texture:SetAllPoints()
	frame.texture:SetTexture(unpack(AnchorFrameColor))
	
	local text = frame:CreateFontString(nil, nil, "GameFontNormal")
	text:SetText(FrameRealNames[frameName])
	text:SetPoint("CENTER")
	
	FrameAnchors[frameName] = frame
end

function mMinimap:SetAllFramePositions()
	self = mMinimap
	for frameName, frame in pairs(MultiFrames) do
		self:SetFramePosition(frameName)
	end
end

function mMinimap:SetFramePosition(frameName, x, y)
	db = self.db.profile
	local pos = db.FramePositions[frameName]
	if not pos then return end
	pos[1] = x or pos[1]
	pos[2] = x or pos[2]
	x = x or pos[1]
	y = y or pos[2]	
	if not x or not y then return end
		
	local anchor = FrameAnchors[frameName]
	
	SettingPositions = true
	if frameName == "Capture" then
		for i = 1, NUM_EXTENDED_UI_FRAMES do
			_G["WorldStateCaptureBar" .. i]:ClearAllPoints()
			_G["WorldStateCaptureBar" .. i]:SetPoint("CENTER", UIParent, "CENTER", x, y)
		end
		if anchor then
			anchor:ClearAllPoints()
			anchor:SetPoint("CENTER", UIParent, "CENTER", x, y)
		end
	else
		MultiFrames[frameName]:ClearAllPoints()
		MultiFrames[frameName]:SetPoint("CENTER", UIParent, "CENTER", x, y)
		
		if frameName == "WatchFrame" then
			local wideFrame = GetCVar("watchFrameWidth")
			if wideFrame == "1" then
				WatchFrame:SetWidth(db.WatchFrameWidth + 100)
			end
			WatchFrame:SetHeight(db.WatchFrameHeight)
		end
		
		if anchor then
			anchor:ClearAllPoints()
			if MultiFrames[frameName].SetPoint then
				anchor:SetAllPoints(MultiFrames[frameName])
			else
				anchor:SetPoint("CENTER", UIParent, "CENTER", x, y)
			end
		end
	end
	SettingPositions = false
end

function mMinimap:HookedSetPoint(...)
	local frame = ...
	if not SettingPositions and frame then
		self:SetFramePosition(frame.GetName and frame:GetName() or "")
	end
end

function mMinimap:HookedCaptureSetPoint()
	if not SettingPositions then
		self:SetFramePosition("Capture")
	end
end

function mMinimap:WorldStateAlwaysUpFrame_Update(...)
	while NumHookedCaptureBars < NUM_EXTENDED_UI_FRAMES do
		NumHookedCaptureBars = NumHookedCaptureBars + 1
		self:SecureHook(_G["WorldStateCaptureBar" .. NumHookedCaptureBars], "SetPoint", "HookedCaptureSetPoint")
		self:HookedCaptureSetPoint()
	end
end